@@ -1,10 +1,10 @@
 /* Test program, used by the plural-1 test.
-   Copyright (C) 2001-2002 Free Software Foundation, Inc.
+   Copyright (C) 2001-2002, 2009 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,8 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -28,6 +27,11 @@
 #undef _LIBINTL_H
 #include "libgnuintl.h"
 
+/* Disable the override of setlocale that libgnuintl.h activates on MacOS X
+   and Windows.  This test relies on the fake setlocale function in
+   setlocale.c.  */
+#undef setlocale
+
 int
 main (int argc, char *argv[])
 {
@@ -38,7 +42,7 @@ main (int argc, char *argv[])
     return 1;
 
   textdomain ("cake");
-  bindtextdomain ("cake", ".");
+  bindtextdomain ("cake", "plural-1-dir");
   printf (ngettext ("a piece of cake", "%d pieces of cake", n), n);
   printf ("\n");
   return 0;
